--[[

HonorConquestText Addon

using aceaddon: http://www.wowace.com/addons/ace3/pages/getting-started/

TODOS:
	Option: fade on combat?
]]

local appName = "HonorConquestText"

HonorConquestText = LibStub("AceAddon-3.0"):NewAddon(appName, "AceEvent-3.0", "AceConsole-3.0");

local mod = HonorConquestText


local defaults = {
    char = {
        locked = false,
        hide_in_combat = false,
        tooltip_in_combat = false,
        positionSaved = false,
        point = nil,
        relativeTo = nil,
        relativePoint = nil,
        xOfs = nil,
        yOfs = nil
    }
}

local appOptions = {
  type = "group",
  args = {
    enable = {
      name = "Enabled",
      desc = "Enables / disables the addon",
      type = "toggle",
      width = "full",
      order = 0,
      set = function(info,val)
          if val then
              mod:Enable();
          else
              mod:Disable();
          end
      end,
      get = function(info) return mod:IsEnabled(); end
    },
    locked = {
      name = "Locked",
      desc = "Locks or unlocks the panel for movement.",
      type = "toggle",
      width = "full",
      order = 1,
      set = function(info,val) mod:setLocked(val); end,
      get = function(info) return mod.db.locked; end
    },
    hide_in_combat = {
      name = "Hide panel in combat.",
      desc = "Hide panel in combat.",
      type = "toggle",
      width = "full",
      order = 2,
      set = function(info,val) mod:setHideInCombat(val); end,
      get = function(info) return mod.db.hide_in_combat; end
    },
    hide_tooltip_in_combat = {
      name = "Hide tooltip in combat.",
      desc = "Hide the Tooltip in combat. Automatically hidden if hiding panel in combat.",
      type = "toggle",
      width = "full",
      order = 3,
      set = function(info,val) mod:setHideTooltipInCombat(val); end,
      get = function(info) return mod.db.hide_tooltip_in_combat; end
    },
    --[[
    hd = {
      name = "Debug Honor",
      desc = "Debug honor capping.",
      type = "toggle",
      set = function(info,val) DEBUG_HONOR_CAPPED = val; mod:updateDisplay(); end,
      get = function(info) return DEBUG_HONOR_CAPPED; end
    },
    cd = {
      name = "Debug Conquest",
      desc = "Debug conquest capping.",
      type = "toggle",
      set = function(info,val) DEBUG_CONQUEST_CAPPED = val; mod:updateDisplay(); end,
      get = function(info) return DEBUG_CONQUEST_CAPPED; end
    },
    ]]
  }
}

local DEBUG = true;
DEBUG_HONOR_CAPPED = false;
DEBUG_CONQUEST_CAPPED = false;
local HONOR_CURRENCY_INDEX = 392;
local CONQUEST_CURRENCY_INDEX = 390;
local IS_INITIALIZED = false;

function debug(msg)
	if DEBUG then
		DEFAULT_CHAT_FRAME:AddMessage(format("HCT: %s", msg));
	end
end

function chat(msg)
	DEFAULT_CHAT_FRAME:AddMessage(format("HonorConquestText: %s", msg));
end


-------------------------------------------------------------------------------
-- Initialize
-------------------------------------------------------------------------------
function mod:OnInitialize()
    -- Initialize database
    self._db = LibStub("AceDB-3.0"):New("HonorConquestTextDB", defaults, "Default");
    self.db = self._db.char;

    self:SetupOptions();
end


-------------------------------------------------------------------------------
-- Enable/disable
-------------------------------------------------------------------------------
function mod:OnEnable()
    self:RegisterEvent("ADDON_LOADED");
    self:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
    self:RegisterEvent("PLAYER_ENTERING_WORLD");
    self:RegisterEvent("PLAYER_REGEN_DISABLED");
    self:RegisterEvent("PLAYER_REGEN_ENABLED");

    if IS_INITIALIZED then
        self.main_frame:Show();
    else
        self:setupDisplay();
    end
end

function mod:OnDisable()
    self.main_frame:Hide();
end


-------------------------------------------------------------------------------
-- EVENTS
-------------------------------------------------------------------------------
function mod:ADDON_LOADED()
    chat("Addon loaded.");
    self:updateDisplay();
end

function mod:PLAYER_ENTERING_WORLD()
    debug("Player entering world.");
    self:updateDisplay();
end

function mod:CURRENCY_DISPLAY_UPDATE()
    debug("Currency display updated.");
    self:updateDisplay();
end

function mod:PLAYER_REGEN_DISABLED()
    debug("Player entered combat.");
    if self.db.hide_in_combat then
        debug("Hiding main frame.");
        self.main_frame:Hide();
    end
end

function mod:PLAYER_REGEN_ENABLED()
    debug("Player left combat. Showing main frame.");
    self.main_frame:Show();
end


-------------------------------------------------------------------------------
-- Setup Options
-------------------------------------------------------------------------------
function mod:SetupOptions()
    local AceConfig = LibStub("AceConfig-3.0")
    AceConfig:RegisterOptionsTable(appName, appOptions, {"/hct"})


    -- Use AceConsole-3.0 to register a Chat Command
    self:RegisterChatCommand("hct", "ChatCommand");

    local gui = LibStub("AceConfigDialog-3.0");
    gui:SetDefaultSize(appName, 475, 225);
    gui:AddToBlizOptions(appName, appName);
end

function mod:ChatCommand(input)
    if not input or input:trim() == "" then
        LibStub("AceConfigDialog-3.0"):Open(appName)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand('hct', appName, input)
    end
end


-------------------------------------------------------------------------------
-- Setup Display
-------------------------------------------------------------------------------
--[[

(HONOR CAPPED)
RATING:   ####
TO CAP:   ####
CONQUEST: ####

--]]
function mod:setupDisplay()

    local childrenMouseEnabled = false;

    -- MAIN FRAME
    self.main_frame = CreateFrame("Frame", "HCT_main_frame", UIParent);
    self.main_frame:SetFrameStrata("HIGH")
    self.main_frame:SetWidth(160)
    -- Height will be set later.
    self:loadPosition();
    self.main_frame:SetBackdrop({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        tile = true,
        tileSize = 32

    });
    self:setLocked(self.db.locked);


    --[[
    CAPPED HONOR
    CAPPED CONQ | TO CAP: ####
    CONQUEST: ####
    RATING: ####
    
    --]]
  


    -- HONOR CAPPED
    debug("Settig up CAPPED HONOR.");
    local HC_REF_TO = self.main_frame;
    self.frame_honor_capped = CreateFrame("Frame", "HCT_frame_honor_capped", self.main_frame);
    self.frame_honor_capped:EnableMouse(childrenMouseEnabled)
    self.frame_honor_capped:SetMovable(childrenMouseEnabled)
    self.frame_honor_capped:SetPoint("LEFT", self.main_frame, "LEFT");
    self.frame_honor_capped:SetPoint("RIGHT", self.main_frame, "RIGHT");
    self.frame_honor_capped:SetPoint("TOP", HC_REF_TO, "TOP", 0, -5);
    locked = {
      name = "Locked",
      desc = "Locks or unlocks the panel for movement.",
      type = "toggle",
      set = function(info,val) mod:setLocked(val); end,
      get = function(info) return mod.db.locked; end
    },
    self.frame_honor_capped:SetHeight(20);

        self.frame_honor_capped_label = self.frame_honor_capped:CreateFontString(nil, "OVERLAY", "GameFontWhite");
        self.frame_honor_capped_label:SetJustifyH("CENTER");
        self.frame_honor_capped_label:SetJustifyV("MIDDLE");
        self.frame_honor_capped_label:SetPoint("LEFT", 11, 0);
        self.frame_honor_capped_label:SetPoint("RIGHT", -12, 0);
        self.frame_honor_capped_label:SetPoint("TOP", 0, 0);
        self.frame_honor_capped_label:SetText(format("|cffff0000CAPPED HONOR|r"));



  
    -- CONQUEST CAPPED
    debug("Setting up CAPPED CONQUEST");
    local CONQ_REF_TO = self.frame_honor_capped;
    self.conquest = CreateFrame("Frame", "HCT_conquest_currency", self.main_frame);
    self.conquest:EnableMouse(childrenMouseEnabled);
    self.conquest:SetMovable(childrenMouseEnabled);
    self.conquest:SetPoint("LEFT", self.main_frame, "LEFT");
    self.conquest:SetPoint("RIGHT", self.main_frame, "RIGHT");
    self.conquest:SetPoint("TOP", CONQ_REF_TO, "BOTTOM");
    self.conquest:SetHeight(20);

        -- CONQUEST CAPPED
        self.frame_conquest_capped = CreateFrame("Frame", "HCT_frame_conquest_capped", self.main_frame);
        self.frame_conquest_capped:EnableMouse(childrenMouseEnabled)
        self.frame_conquest_capped:SetMovable(childrenMouseEnabled)
        self.frame_conquest_capped:SetPoint("LEFT", self.main_frame, "LEFT");
        self.frame_conquest_capped:SetPoint("RIGHT", self.main_frame, "RIGHT");
        self.frame_conquest_capped:SetPoint("TOP", self.conquest, "TOP");
        self.frame_conquest_capped:SetHeight(20);

            self.frame_conquest_capped_label = self.frame_conquest_capped:CreateFontString(nil, "OVERLAY", "GameFontWhite");
            self.frame_conquest_capped_label:SetJustifyH("CENTER");
            self.frame_conquest_capped_label:SetJustifyV("MIDDLE");
            self.frame_conquest_capped_label:SetPoint("LEFT", 11, 0);
            self.frame_conquest_capped_label:SetPoint("RIGHT", -12, 0);
            self.frame_conquest_capped_label:SetPoint("TOP", 0, 0);
            self.frame_conquest_capped_label:SetText(format("|cffff0000CAPPED CONQ|r")); 



        debug("Setting up TO CAP.");
        -- PERSONAL to_cap
        self.frame_to_cap = CreateFrame("Frame", "HCT_frame_to_cap", self.main_frame);
        self.frame_to_cap:EnableMouse(childrenMouseEnabled)
        self.frame_to_cap:SetMovable(childrenMouseEnabled)
        self.frame_to_cap:SetPoint("LEFT", self.main_frame, "LEFT");
        self.frame_to_cap:SetPoint("RIGHT", self.main_frame, "RIGHT");
        self.frame_to_cap:SetPoint("TOP", self.conquest, "TOP");
        self.frame_to_cap:SetHeight(20);

            self.frame_to_cap_label = self.frame_to_cap:CreateFontString(nil, "OVERLAY", "GameFontWhite");
            self.frame_to_cap_label:SetJustifyH("LEFT");
            self.frame_to_cap_label:SetJustifyV("MIDDLE");
            self.frame_to_cap_label:SetPoint("LEFT", 11, 0);
            self.frame_to_cap_label:SetPoint("RIGHT", -12, 0);
            self.frame_to_cap_label:SetPoint("TOP", 0, 0);
            self.frame_to_cap_label:SetText("TO CAP:");

            self.frame_to_cap_value = self.frame_to_cap:CreateFontString(nil, "OVERLAY", "GameFontWhite");
            self.frame_to_cap_value:SetJustifyH("RIGHT");
            self.frame_to_cap_value:SetJustifyV("MIDDLE");
            self.frame_to_cap_value:SetPoint("LEFT", 11, 0);
            self.frame_to_cap_value:SetPoint("RIGHT", -12, 0);
            self.frame_to_cap_value:SetPoint("TOP", 0, 0);
            self.frame_to_cap_value:SetText("");




    -- PERSONAL CONQUEST
    debug("Setting up CONQUEST.");
    local C_REF_TO = self.conquest;
    self.frame_conquest = CreateFrame("Frame", "HCT_frame_conquest", self.main_frame);
    self.frame_conquest:EnableMouse(childrenMouseEnabled)
    self.frame_conquest:SetMovable(childrenMouseEnabled)
    self.frame_conquest:SetPoint("LEFT", self.main_frame, "LEFT");
    self.frame_conquest:SetPoint("RIGHT", self.main_frame, "RIGHT");
    self.frame_conquest:SetPoint("TOP", C_REF_TO, "BOTTOM");
    self.frame_conquest:SetHeight(20);
    self.frame_conquest:SetScript("OnEnter", function()
        if self.db.hide_tooltip_in_combat and UnitAffectingCombat("player") then
            debug("Not showing tooltip while in combat.");
        else
            GameTooltip:SetOwner(self.main_frame, "ANCHOR_TOPLEFT");
            GameTooltip:SetCurrencyByID(CONQUEST_CURRENCY_INDEX);
        end
    end);
    self.frame_conquest:SetScript("OnLeave", function()
        GameTooltip:Hide();
    end);

        self.frame_conquest_label = self.frame_conquest:CreateFontString(nil, "OVERLAY", "GameFontWhite");
        self.frame_conquest_label:SetJustifyH("LEFT");
        self.frame_conquest_label:SetJustifyV("MIDDLE");
        self.frame_conquest_label:SetPoint("LEFT", 11, 0);
        self.frame_conquest_label:SetPoint("RIGHT", -12, 0);
        self.frame_conquest_label:SetPoint("TOP", 0, 0);
        self.frame_conquest_label:SetText("CONQUEST:");

        self.frame_conquest_value = self.frame_conquest:CreateFontString(nil, "OVERLAY", "GameFontWhite");
        self.frame_conquest_value:SetJustifyH("RIGHT");
        self.frame_conquest_value:SetJustifyV("MIDDLE");
        self.frame_conquest_value:SetPoint("LEFT", 11, 0);
        self.frame_conquest_value:SetPoint("RIGHT", -12, 0);
        self.frame_conquest_value:SetPoint("TOP", 0, 0);
        self.frame_conquest_value:SetText("");




    -- PERSONAL RATING
    debug("Setting up PERSONAL RATING.");
    local RATING_REF_TO = self.frame_conquest;
    self.frame_rating = CreateFrame("Frame", "HCT_frame_rating", self.main_frame);
    self.frame_rating:EnableMouse(childrenMouseEnabled)
    self.frame_rating:SetMovable(childrenMouseEnabled)
    self.frame_rating:SetPoint("LEFT", self.main_frame, "LEFT");
    self.frame_rating:SetPoint("RIGHT", self.main_frame, "RIGHT");
    self.frame_rating:SetPoint("TOP", RATING_REF_TO, "BOTTOM");
    self.frame_rating:SetHeight(20);

        self.frame_rating_label = self.frame_rating:CreateFontString(nil, "OVERLAY", "GameFontWhite");
        self.frame_rating_label:SetJustifyH("LEFT");
        self.frame_rating_label:SetJustifyV("MIDDLE");
        self.frame_rating_label:SetPoint("LEFT", 11, 0);
        self.frame_rating_label:SetPoint("RIGHT", -12, 0);
        self.frame_rating_label:SetPoint("TOP", 0, 0);
        self.frame_rating_label:SetText("RATING:");

        self.frame_rating_value = self.frame_rating:CreateFontString(nil, "OVERLAY", "GameFontWhite");
        self.frame_rating_value:SetJustifyH("RIGHT");
        self.frame_rating_value:SetJustifyV("MIDDLE");
        self.frame_rating_value:SetPoint("LEFT", 11, 0);
        self.frame_rating_value:SetPoint("RIGHT", -12, 0);
        self.frame_rating_value:SetPoint("TOP", 0, 0);
        self.frame_rating_value:SetText("");


    self:updateDisplay()
    
end


function mod:resetPosition()
    debug("Resetting frame position.");
    self.main_frame:ClearAllPoints();
    self.main_frame:SetPoint("CENTER", UIParent, "CENTER");
end

function mod:loadPosition()
    debug("Loading frame position.");
    if self.db.positionSaved then
        self.main_frame:SetPoint(
            self.db.point,
            self.db.relativeTo,
            self.db.relativePoint,
            self.db.xOfs,
            self.db.yOfs
        );
    else
        self:resetPosition();
    end
end


function mod:setHideInCombat(state)
    self.db.hide_in_combat = state;
    if self.db.hide_in_combat and UnitAffectingCombat("player") then
        self.main_frame:Hide();
    else
        self.main_frame:Show();
    end
end

function mod:setHideTooltipInCombat(state)
    self.db.hide_tooltip_in_combat = state;
end


function mod:setPosition()
    debug("Setting frame position.");
    self.main_frame:SetUserPlaced(true);
    self.db.positionSaved = true;
    point, relativeTo, relativePoint, xOfs, yOfs = self.main_frame:GetPoint()
    self.db.point = point;
    self.db.relativeTo = relativeTo;
    self.db.relativePoint = relativePoint;
    self.db.xOfs = xOfs;
    self.db.yOfs = yOfs;
end


function mod:setLocked(state)
    self.db.locked = state;
    if self.db.locked then
        debug("Locking frame.");
        self.main_frame:EnableMouse(false);
        self.main_frame:SetMovable(false);
        self.main_frame:RegisterForDrag();
    else
        debug("Unlocking frame.");
        self.main_frame:EnableMouse(true)
        self.main_frame:SetMovable(true)
        -- http://wowprogramming.com/docs/widgets/Frame/RegisterForDrag
        self.main_frame:RegisterForDrag("LeftButton");
        self.main_frame:SetScript("OnDragStart", self.Drag);
        self.main_frame:SetScript("OnDragStop", self.StopDrag);
    end
end

function mod.Drag()
    debug("Starting drag.");
    mod.main_frame:StartMoving();
end

function mod.StopDrag()
    debug("Stopping drag.");
    mod.main_frame:StopMovingOrSizing();
    mod:setPosition();
end

function mod:updateFrameHeight(hcapped)
    height = 0
    if hcapped then
        -- need to also set point for rating frame to top of the main frame.
        self.conquest:SetPoint("TOP", self.frame_honor_capped, "BOTTOM");
        height = 80;
    else
        self.conquest:SetPoint("TOP", self.main_frame, "TOP", 0, -5);
        height = 60;
    end
    self.main_frame:SetHeight(height + 5);
end

function mod:isHonorCapped()
    name, amount, texture, earnedThisWeek, weeklyMax, totalMax, isDiscovered = GetCurrencyInfo(HONOR_CURRENCY_INDEX)
    return amount >= 4000 or DEBUG_HONOR_CAPPED;
end


function mod:SetToCap()
    pointsThisWeek, maxPointsThisWeek, tier2Quantity, tier2Limit, tier1Quantity, tier1Limit = GetPVPRewards();
    local to_cap = maxPointsThisWeek - pointsThisWeek

    self.frame_to_cap_value:SetText(to_cap);

    if to_cap <= 0 then
        self.frame_to_cap:Hide();
        self.frame_conquest_capped:Show();
    else
        self.frame_conquest_capped:Hide();
        self.frame_to_cap:Show();
    end

    if DEBUG_CONQUEST_CAPPED then
        self.frame_to_cap:Hide();
        self.frame_conquest_capped:Show();
    end


end

function mod:SetConquest()
    name, amount, texture, earnedThisWeek, weeklyMax, totalMax, isDiscovered = GetCurrencyInfo(CONQUEST_CURRENCY_INDEX)
    self.frame_conquest_value:SetText(amount);
end

function mod:SetRating()
    personalBGRating, ratedBGreward, pointsThisWeek, maxPointsThisWeek, unknown, unknown, weeklyWins, weeklyPlayed, unknown, unknown = GetPersonalRatedBGInfo();
    self.frame_rating_value:SetText(personalBGRating);
end

function mod:updateDisplay()
    -- Is Honor capped?
    local hcapped = self:isHonorCapped()
    if hcapped then
        debug("Showing HONOR CAPPED.")
        self.frame_honor_capped:Show();
    else
        debug("Hiding HONOR CAPPED.")
        self.frame_honor_capped:Hide();
    end

    self:SetToCap();
    self:SetConquest();
    self:SetRating();
    -- set to cap
    -- set conquest

    self:updateFrameHeight(hcapped);
end




--=============================================================================
--=============================================================================
--=============================================================================
--=============================================================================
--=============================================================================
--=============================================================================
--=============================================================================

